%% Radial cardiac IRGN-T(G)V [1] reconstruction:
% 32 channel cardiac compressed to virtual 12 channels, 25 radial
% projections.
%
% Some experiments from [1] can be reproduced with this code. To 
% switch between L2, TV and TGV regularization, just change the value of 
% the variable params.tvtype. In the paper, two different regularization
% parameters were used. In the case of moderate undersampling, the final 
% regularization parameter is reduced to zero. A more pronounced T(G)V
% filtering effect in the case of high undersampling can be achieved by
% stopping this decrease during the iterations. This can be controlled by
% setting the variable params.beta_min either to 0 or to, e.g., 5e-3.
%
% This code requires the NUFFT [2,3] toolbox developed by Jeff Fessler and
% his students and an interface framework developed by Miki Lustig.
% In order to make the code self-contained, the necessary functions are 
% included with permission of the original authors.
% However, please note that the NUFFT toolbox is updated regularly, and the
% newest version can always be obtained at:
%
% The cardiac data was obtained with a measurement protocol for real-time
% imaging without cardiac gating and during free breathing [4,5].
% 
% http://www.eecs.umich.edu/~fessler/irt/irt/
% 
% References:
% [1] Knoll, F.; Clason, C.; Bredies, K.; Uecker, M.; Stollberger, R.:
% Parallel imaging with nonlinear reconstruction using variational
% penalties.
% Magnetic Resonance in Medicine, Early View: DOI: 10.1002/mrm.22964.
% 
% [2] Fessler, J. A. and Sutton, B. P.:
% Nonuniform fast Fourier transforms using min-max interpolation. IEEE
% Transactions on Signal Processing 51:560-574 (2003).
%
% [3] Sutton, B. P.; Noll, D. C.; Fessler, J. A:
% Fast, iterative image reconstruction for MRI in the presence of field
% inhomogeneities. IEEE Trans Med Imaging 22:178-188 (2003).
%
% [4] Zhang, S.; Block, K.T.; Frahm, J.:
% Magnetic resonance imaging in real time: advances using radial FLASH.
% Journal of Magnetic Resonance Imaging 31:101-109 (2010)
% 
% [5] Uecker, M.; Zhang, S.; Frahm, J.:
% Nonlinear inverse reconstruction for real-time MRI of the human heart
% using undersampled radial FLASH
% Magnetic Resonance in Medicine 63:1456-1462 (2010)

% Christian Clason and Florian Knoll, July 2011
% christian.clason@uni-graz.at
% http://www.uni-graz.at/~clason 
% florian.knoll@tugraz.at
% http://www.imt.tugraz.at/research/mr-image-reconstruction

clear all; close all; clc;
addpath(genpath('NUFFT')); %#ok<MCAP>

%% Reconstruction parameters
params.maxit    = 6;    % maximum number of IRGN iterations
params.tvtype   = 1;    % regularization term: 0: L2, 1: TV, 2: TGV
params.beta_min = 5e-3; % final value of beta: 0: no T(G)V effect, >0: effect

%% Load data
load radial_cardiac_25_projections;
[nFE,nSpokes,nCh] = size(rawdata);

%% Generate sampling operator
% simple density compensation
rho = linspace(-0.5,0.5,nFE)';
w   = abs(rho)*nFE/nSpokes;
w   = repmat(w, [1, nSpokes]);

% generate Fourier sampling operator
FT   = NUFFT(k, w, 1, 0, [nFE,nFE], 2);
nft  = @(x) FT*x;
inft = @(x) FT'*x;

%% SOS image    
img_sos = inft(rawdata);
img_sos = sqrt(sum(abs(img_sos).^2,3));

% zoom and rotate
zoomPos = [nFE/4+0:nFE/2+nFE/4+0; nFE/4:nFE/2+nFE/4];
img_sos_disp = img_sos(zoomPos(1,:),zoomPos(2,:),end);
img_sos_disp = abs(rot90(img_sos_disp',2));
img_sos_disp = img_sos_disp/max(img_sos_disp(:));

figure, imshow(img_sos_disp,[]); title('SOS with DensComp');

%% IRGN-T(G)V reconstruction
% usually there should be no need to change these
params.alpha0    = 1;      % initial penalty alpha_0 (L2, sensitivites)
params.beta0     = 2;      % initial penalty beta_0 (TV, image)
params.alpha_min = 0;      % final value of alpha
params.alpha_q   = 1/10;   % reduction factor for alpha
params.beta_q 	 = 1/10;   % reduction factor for beta
params.tvits     = 20;     % initial number of gradient steps

tic
disp('-------------------------');
disp('Starting IRGN-T(G)V');
disp('-------------------------');
imgIRGNTV = irgn_solve(rawdata,nft,inft,params);
disp(['Time: ', num2str(toc/60), ' min']);

%% Display results
brightening = 0.3;
windowing   = 0.8;

% zoom and rotate
imgIRGNTV_disp = imgIRGNTV(zoomPos(1,:),zoomPos(2,:),end);
imgIRGNTV_disp = abs(rot90(imgIRGNTV_disp',2));
imgIRGNTV_disp = imgIRGNTV_disp/max(imgIRGNTV_disp(:));

figure, imshow(brighten(imgIRGNTV_disp,brightening), [0 windowing]); 
title('IRGN');
